<?php
namespace Drupal\translationcopilot\Drush\Commands;


use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\locale\Entity\Locale;
use Drupal\locale\Entity\Translation;

use Drush\Commands\DrushCommands;
use Drupal\Core\Language\Language;
use Drupal\language\Entity\ConfigurableLanguage;
use Drupal\taxonomy\Entity\Term;
use Drupal\Core\Language\LanguageInterface;
use Drupal\block_content\Entity\BlockContent;
use Drupal\menu_link_content\Entity\MenuLinkContent;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\paragraphs\Entity\Paragraph;

class TranslationcopilotCommands extends DrushCommands {
	use StringTranslationTrait;

    /**
     * Adds a new language.
     *
	 * @command locale:add-language
	 * @option lang-code The name of the language to use.
	 * @option lang-name The name of the language to use.
     * @aliases lal
	 *
     * @usage locale:add-language --lang-code=fr --lang-name=French
     *   Adds French language to the site.
     */
    public function addLanguage($options = ['lang-code' => '','lang-name' => '']) {
		$langcode = $options['lang-code'];
		$langname = $options['lang-name'];
		
        $standard_languages = \Drupal::languageManager()->getStandardLanguageList();
		$label = $standard_languages[$langcode][0];
		$direction = $standard_languages[$langcode][2] ?? ConfigurableLanguage::DIRECTION_LTR;

		$entity = \Drupal::entityTypeManager()->getStorage('configurable_language')->create();
		$entity->set('id', $langcode);
		$entity->set('label', $label);
		$entity->set('direction', $direction);
		$languages = \Drupal::languageManager()->getLanguages(ConfigurableLanguage::STATE_CONFIGURABLE);
		$last_language = end($languages);
		$entity->setWeight($last_language->getWeight() + 1);
		$entity->save();

        $this->output()->writeln($this->t('Language "@name" (code: @langcode) added successfully.', [
            '@name' => $langname,
            '@langcode' => $langcode,
        ]));
    }
	
	/**
     * Adds a translation for a specific string.
     *
     *
     * @command locale:add-translation
     * @aliases lat
     * @usage locale:add-translation --source-lang=en --dest-lang=fr
     *   Adds a translation for provided language.
     */
    public function addTranslation($options = ['source-lang' => '','dest-lang' => '']) {
		$sourcelang = $options['source-lang'];
		
		$destlang = $options['dest-lang'];
		
		$standard_languages = \Drupal::languageManager()->getStandardLanguageList();
		$langname = $standard_languages[$destlang][0];
				
		
        $nids  = \Drupal::entityQuery("node")
		->accessCheck(TRUE)
		->condition('langcode', $sourcelang)
		//->condition('type', 'my_custom_type') //use this for specific content types
		->execute();
		$nodes =  \Drupal\node\Entity\Node::loadMultiple($nids);
		
		foreach ($nodes as $node) {
		  $title = $this->textTranslation($node->getTitle(),$langname);
		  $value = $this->textTranslation($node->get('body')->value,$langname);
		  //Creating the translation Spanish in this case
		  try
		  {
			$node_es = $node->getTranslation($destlang);
		  }
		  catch(\Exception $e)
		  {
			$node_es = $node->addTranslation($destlang);
		  }
		  //$node_es = $node->addTranslation($destlang);
		  $node_es->title = $title;
		  $node_es->body->value = $value;
		  $node_es->body->format = 'full_html';
		  //Saving the node
		  $node_es->save();
		}
		
		
		//Load all taxonomies
		
		$vocabularies = \Drupal::entityTypeManager()->getStorage('taxonomy_vocabulary')->loadMultiple();
		
        foreach ($vocabularies as $vocabulary) {
            // Load all terms in the vocabulary.
            $terms = \Drupal::entityTypeManager()->getStorage('taxonomy_term')->loadByProperties(['vid' => $vocabulary->id()]);
			
            foreach ($terms as $term) {
				
				$translated_name = '';
				if($term->getName() != '')
				{
					$translated_name = $this->textTranslation($term->getName(),$langname);
				}
				$translated_description = '';
				if($term->getDescription() != '')
				{
					$translated_description = $this->textTranslation($term->getDescription(),$langname);
				}
				
                // Example: Update translation for French.
                $this->updateTermTranslation($term, $destlang, $translated_name,$translated_description);
            }
        }
		
		
		// Load all block content entities.
		
        $block_contents = \Drupal::entityTypeManager()->getStorage('block_content')->loadMultiple();
		
        foreach ($block_contents as $block_content) {
			
            // Example: Add translation for French.
			$translated_title = $this->textTranslation($block_content->label(),$langname);
			$translated_description = $this->textTranslation($block_content->get('body')->value,$langname);
            $this->updateBlockTranslation($block_content, $destlang, $translated_title, $translated_description);
        }
				
		// Load all Menus entities
		
		$menu_definitions = \Drupal::entityTypeManager()->getStorage('menu')->loadMultiple();
		foreach ($menu_definitions as $menu) {
			$menu_name = $menu->Id();
			$menu_links = \Drupal::entityTypeManager()
			  ->getStorage('menu_link_content')
			  ->loadByProperties(['menu_name' => $menu_name]);		
			
			
			foreach ($menu_links as $menu_link) {
				
				$translated_url = 'internal:/'; // Your translated URL (optional).
				$translated_menu_title = $this->textTranslation($menu_link->getTitle(),$langname);
				$this->add_menu_link_translation($menu_link->Id(), $destlang, $translated_menu_title, $translated_url);
			}
			
		}
		
		// Load all paragraph content entities.
		
        $paragraph_contents = \Drupal::entityTypeManager()->getStorage('paragraph')->loadMultiple();
		
        foreach ($paragraph_contents as $paragraph_content) {
			
            // Example: Add translation for French.
			$translated_title = $this->textTranslation($paragraph_content->label(),$langname);
			$translated_description = $this->textTranslation($paragraph_content->get('body')->value,$langname);
            $this->updateParagraphTranslation($paragraph_content, $destlang, $translated_title, $translated_description);
        }
		
		
    }
	
	public function textTranslation($text,$langname)
	{
		try {
		  $config = \Drupal::getContainer()
		  ->get('config.factory')
		  ->get('translationcopilot.settings');
		  $curl = curl_init();

		  curl_setopt_array($curl, array(
		  CURLOPT_URL => 'https://generativelanguage.googleapis.com/v1beta/models/gemini-1.5-flash:generateContent?key='.$config->get('translationcopilot_api_key'),
		  CURLOPT_RETURNTRANSFER => true,
		  CURLOPT_ENCODING => '',
		  CURLOPT_MAXREDIRS => 10,
		  CURLOPT_TIMEOUT => 0,
		  CURLOPT_FOLLOWLOCATION => true,
		  CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
		  CURLOPT_SSL_VERIFYPEER => false,
		  CURLOPT_CUSTOMREQUEST => 'POST',
		  CURLOPT_POSTFIELDS =>'{
			"contents": [
					{
						"parts":[
							{
								"text": "Translate the following text to '.$langname.': '.$text.'"
							}
						]
					}
				]
			}',
		  CURLOPT_HTTPHEADER => array(
				'Content-Type: application/json'
			  ),
			));
		  
		  $response = json_decode(curl_exec($curl));
		  		  
		  $translation = htmlspecialchars_decode($response->candidates[0]->content->parts[0]->text);
		  
		}
		catch (RequestException $e) {
		  $this->getLogger('translationcopilot')->error('Translation copilot error: @error', ['@error' => $e->getMessage()]);
		  $this->getMessages($e->getMessage());
		}
		return $translation;
	}
	
	public function updateTermTranslation(Term $term, $langcode, $translated_name, $translated_description) {
		
        // Check if the term has a translation in the specified language.
        if ($term->hasTranslation($langcode)) {
            // Get the translation.
            $translated_term = $term->getTranslation($langcode);
            // Update the name.
            $translated_term->setName($translated_name);
        }
		else
		{
			// Add a new translation.
            $translated_term = $term->addTranslation($langcode, [
                'name' => $translated_name,
                'description' => $translated_description, // Copy other fields if needed.
                // Add more fields here as necessary.
            ]);          
		}
		
		try
	    {
			// Save the new translation.
			$translated_term->save();
		}
		catch(\Exception $e)
		{
			echo $e->getMessage();
		}
    }
	
	public function updateBlockTranslation(BlockContent $block_content, $langcode, $translated_title, $translated_description) {
        // Check if the block already has a translation in the specified language.
        if ($block_content->hasTranslation($langcode)) {
            // Get the translation.
            $translated_block = $block_content->getTranslation($langcode);
            // Update the title.
            $translated_block->set('info', $translated_title);
			$translated_block->set('body', $translated_description);
        }
		else
		{
			// Create the new translation.
            $translated_block = $block_content->addTranslation($langcode, [
                'info' => $translated_title,
                'body' => $translated_description, // Copy other fields as necessary.
                // Add additional fields if needed.
            ]);  
		}
		try
	    {
			// Save the new translation.
			$translated_block->save();
		}
		catch(\Exception $e)
		{
			echo $e->getMessage();
		}
		
    }
	
	public function add_menu_link_translation($menu_link_id, $language_code, $translated_title, $translated_url = NULL) {
	  // Load the menu link entity.
	  $menu_link = MenuLinkContent::load($menu_link_id);
	  
	  if (!$menu_link) {
		print_r('Menu link not found.');
		return;
	  }

	  // Create or load the translation.
	  // $translated_link = $menu_link->getTranslation($language_code);
	  
	  try
	  {
		$translated_link = $menu_link->getTranslation($language_code, [
			'title' => $translated_title,
		]);
	  }
	  catch(\Exception $e)
	  {
		// If it doesn't exist, create a new translation.
		$translated_link = $menu_link->addTranslation($language_code, [
			'title' => $translated_title,
		]);
	  }
	   // Save the new translation.
	  $translated_link->save();
	}
	
	
	public function updateParagraphTranslation(Paragraph $paragraph_content, $langcode, $translated_title, $translated_description) {
        // Check if the block already has a translation in the specified language.
        if ($paragraph_content->hasTranslation($langcode)) {
            // Get the translation.
            $translated_paragraph = $paragraph_content->getTranslation($langcode);
            // Update the title.
            $translated_paragraph->set('info', $translated_title);
			$translated_paragraph->set('body', $translated_description);
        }
		else
		{
			// Create the new translation.
            $translated_paragraph = $paragraph_content->addTranslation($langcode, [
                'info' => $translated_title,
                'body' => $translated_description, // Copy other fields as necessary.
                // Add additional fields if needed.
            ]);  
		}
		try
	    {
			// Save the new translation.
			$translated_paragraph->save();
		}
		catch(\Exception $e)
		{
			echo $e->getMessage();
		}
		
    }
}